/* ---------------------------------------------------------------- */
/* setuid_test.c                                                    */
/* ---------------------------------------------------------------- */
/* Damian Weber http://http://www-crypto.htw-saarland.de/weber/     */
/*                                                                  */
/* program for illustrating various cases of seteuid() and setuid() */
/*                                                                  */
/* ---------------------------------------------------------------- */
/*                                                                  */
/* 1. opening a read-protected file works with EUID=0               */
/* 2. creating a directory with EUID=0 means root owns that dir     */
/* 3. opening a read-protected file does not work with EUID unpriv. */
/* 4. creating a directory with EUID unpriv. shows normal case      */
/* 5. regaining special privileges works as long as not completely  */
/*    dropped with setuid()                                         */
/* 6. completely drop special privileges with setuid()              */
/* 7. regaining special privileges does not work when completely    */
/*    dropped with setuid()                                         */
/*                                                                  */
/* ---------------------------------------------------------------- */

#include <stdio.h>
#include <fcntl.h>

#ifndef __FreeBSD__
#define FILENAME "/etc/wvdial.conf" /* a perm 0400-file (Linux) */
#else
#define FILENAME "/etc/nsmb.conf" /* a perm 0400-file (FreeBSD) */
#endif
#define NEWDIR "NEW_DIRECTORY"
#define UID_UNPRIVILEGED getuid()

#define BUF_LENGTH 10

static void pause(const char *message)
{
        char buffer[BUF_LENGTH];

        puts(message);
        puts("-- press RETURN --");
        fgets(buffer,BUF_LENGTH,stdin);
}

void print_ruid_euid(const char *message)
{
        printf("%s: RUID=%4d, EUID=%4d\n",message,getuid(),geteuid());
}


int main()
{
        int d; /* file descriptor */

        print_ruid_euid("start");

        if ((d=open(FILENAME,O_RDONLY))<0)
                perror("open()");
        pause("file " FILENAME " opened?");
        if (d>0)
        {
                puts("YES");
                close(d);
        }
        else
                puts("NO");

        if (mkdir(NEWDIR,0700)<0)
        {
                perror("mkdir");
                return 1;
        }
        pause("directory created");

        if (rmdir(NEWDIR)<0)
        {
                perror("rmdir");
                return 1;
        }
        pause("directory removed");

        if (seteuid(UID_UNPRIVILEGED)<0)
                perror("seteuid(UID_UNPRIVILEGED)");
        print_ruid_euid("after seteuid(UID_UNPRIVILEGED)");


        if ((d=open(FILENAME,O_RDONLY))<0)
                perror("open()");
        pause("file "FILENAME" opened?");
        if (d>0)
        {
                puts("YES");
                close(d);
        }
        else
                puts("NO");

        if (mkdir(NEWDIR,0700)<0)
        {
                perror("mkdir");
                return 1;
        }
        pause("directory created");

        if (rmdir(NEWDIR)<0)
        {
                perror("rmdir");
                return 1;
        }
        pause("directory removed");

        if (seteuid(0)<0)
                perror("seteuid(0)");
        print_ruid_euid("after seteuid(0)");
        pause("");

        if (setuid(UID_UNPRIVILEGED)<0)
                perror("setuid(UID_UNPRIVILEGED)");
        print_ruid_euid("after setuid(UID_UNPRIVILEGED)");
        pause("");

        if (seteuid(0)<0)
                perror("seteuid(0)");
        print_ruid_euid("after seteuid(0)");

        return 0;

}
